package de.hft.geotime.controllers;

import de.hft.geotime.entities.TimetrackUser;
import de.hft.geotime.entities.projections.UserAllEmbeddedProjection;
import de.hft.geotime.repositories.TimetrackUserRepository;
import org.springframework.data.projection.ProjectionFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class UserController {

    private final TimetrackUserRepository userRepository;
    private final BCryptPasswordEncoder bCryptPasswordEncoder;
    private final ProjectionFactory projectionFactory;

    public UserController(TimetrackUserRepository userRepository, BCryptPasswordEncoder bCryptPasswordEncoder, ProjectionFactory projectionFactory) {
        this.userRepository = userRepository;
        this.bCryptPasswordEncoder = bCryptPasswordEncoder;
        this.projectionFactory = projectionFactory;
    }

    @GetMapping("/whoami")
    public UserAllEmbeddedProjection getUsername(Authentication authentication) {
        TimetrackUser user = userRepository.findFirstByUsername(authentication.getName());
        return projectionFactory.createProjection(UserAllEmbeddedProjection.class, user);
    }

    @PostMapping("/sign-up")
    public ResponseEntity<String> signUp(@RequestBody TimetrackUser user) {
        user.setPassword(bCryptPasswordEncoder.encode(user.getPassword()));
        TimetrackUser byUsername = userRepository.findFirstByUsername(user.getUsername());
        if (byUsername == null) {
            userRepository.save(user);
            return new ResponseEntity<>("Created", HttpStatus.CREATED);
        } else {
            return new ResponseEntity<>("Username already exists!", HttpStatus.CONFLICT);
        }
    }

}
